@@ -1,4 +1,6 @@
+from .app import App, Binding, Parser, SubParser, SubParsers
 from .choices import Choices
+from .param import Param, param
 from .typed_argparse import (
     TypedArgs,
     WithUnionType,
@@ -7,7 +9,7 @@ from .typed_argparse import (
     validate_type_union,
 )
 
-VERSION = "0.1.11"
+VERSION = "0.2.1"
 
 __all__ = [
     "Choices",
@@ -16,4 +18,11 @@ __all__ = [
     "get_choices_from",
     "get_choices_from_class",
     "validate_type_union",
+    "App",
+    "Parser",
+    "SubParser",
+    "SubParsers",
+    "Param",
+    "param",
+    "Binding",
 ]
