@@ -1,6 +1,17 @@
 import enum
 import sys
-from typing import List, Optional, Tuple, TypeVar, Union, cast
+from typing import (
+    Callable,
+    Dict,
+    List,
+    Optional,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+    cast,
+    get_type_hints,
+)
 
 from typing_extensions import Literal
 
@@ -14,6 +25,29 @@ _NoneType = type(None)
 RawTypeAnnotation = object
 
 
+def collect_type_annotations(
+    cls: type, *, include_super_types: bool
+) -> Dict[str, "TypeAnnotation"]:
+    all_annotations: Dict[str, "TypeAnnotation"] = dict()
+
+    # Order by less specific (object) to more specific (actual type) to overwrite
+    # effective annotations
+    types = list(reversed(cls.mro()))
+    if not include_super_types:
+        types = types[-1:]
+
+    for cls in types:
+        if hasattr(cls, "__annotations__"):
+            all_annotations.update(
+                **{
+                    name: TypeAnnotation(annotation)
+                    for name, annotation in get_type_hints(cls).items()
+                }
+            )
+
+    return all_annotations
+
+
 def typename(t: RawTypeAnnotation) -> str:
     if hasattr(t, "__name__"):
         return f"'{getattr(t, '__name__')}'"
@@ -25,7 +59,7 @@ def typename_of(value: object) -> str:
     return typename(type(value))
 
 
-def _get_origin(t: RawTypeAnnotation) -> Optional[object]:
+def _get_origin(t: RawTypeAnnotation) -> Optional[RawTypeAnnotation]:
     return cast(Optional[object], getattr(t, "__origin__", None))
 
 
@@ -44,6 +78,77 @@ class TypeAnnotation:
         self.origin = _get_origin(raw_type)
         self.args = _get_args(raw_type)
 
+    @property
+    def is_bool(self) -> bool:
+        return self.raw_type is bool
+
+    @property
+    def is_optional(self) -> bool:
+        # TODO: Iterate branches of Union to check for None?
+        return self.get_underlying_if_optional() is not None
+
+    def get_underlying_type_converter(self) -> Optional[Union[type, Callable[[object], object]]]:
+        if isinstance(self.raw_type, type) and not issubclass(self.raw_type, enum.Enum):
+            return self.raw_type
+        else:
+            underlying = self.get_underlying_if_optional()
+            if underlying is not None:
+                return underlying.get_underlying_type_converter()
+            underlying = self.get_underlying_if_list()
+            if underlying is not None:
+                return underlying.get_underlying_type_converter()
+
+            allowed_values_if_literal = self.get_allowed_values_if_literal()
+            if allowed_values_if_literal is not None:
+                allowed_values = allowed_values_if_literal
+
+                def converter(x: object) -> object:
+
+                    for allowed_value in allowed_values:
+                        if x == allowed_value:
+                            return allowed_value
+                        else:
+                            try:
+                                x_converted = type(allowed_value)(x)  # type: ignore
+                                if x_converted == allowed_value:
+                                    return allowed_value
+                            except (ValueError, TypeError):
+                                pass
+
+                    # Here we could raise a TypeError or ValueError, but it looks like relying
+                    # on `choices` instead actually produces a better error message.
+                    return x
+
+                return converter
+
+            allowed_values_if_emum = self.get_allowed_values_if_enum()
+            if (
+                allowed_values_if_emum is not None
+                and isinstance(self.raw_type, type)
+                and issubclass(self.raw_type, enum.Enum)
+            ):
+                enum_type: Type[enum.Enum] = self.raw_type
+                allowed_values = allowed_values_if_emum
+
+                def converter(x: object) -> object:
+
+                    for allowed_value in allowed_values:
+                        if x == allowed_value or x == allowed_value.value:  # type: ignore
+                            return allowed_value
+                        else:
+                            try:
+                                return enum_type[x]  # type: ignore
+                            except KeyError:
+                                pass
+
+                    # Here we could raise a TypeError or ValueError, but it looks like relying
+                    # on `choices` instead actually produces a better error message.
+                    return x
+
+                return converter
+
+            return None
+
     def get_underlying_if_optional(self) -> Optional["TypeAnnotation"]:
         if self.origin is Union and len(self.args) == 2 and _NoneType in self.args:
             for t in self.args:
@@ -108,7 +213,7 @@ class TypeAnnotation:
         underlying_if_optional = self.get_underlying_if_optional()
         if underlying_if_optional is not None:
             if value is None:
-                return value, None
+                return None, None
             else:
                 return underlying_if_optional.validate(value)
 
@@ -192,19 +297,18 @@ class TypeAnnotation:
                 f"value is of type {typename_of(value)}, expected {typename(self.raw_type)}",
             )
 
+    def validate_with_error(
+        self,
+        value: object,
+        arg_name: str,
+    ) -> object:
 
-def validate_value_against_type(
-    arg_name: str,
-    value: object,
-    raw_type_annotation: RawTypeAnnotation,
-) -> object:
+        value, error = self.validate(value)
 
-    value, error = TypeAnnotation(raw_type_annotation).validate(value)
+        if error is not None:
+            raise TypeError(f"Failed to validate argument '{arg_name}': {error}")
 
-    if error is not None:
-        raise TypeError(f"Failed to validate argument '{arg_name}': {error}")
-
-    return value
+        return value
 
 
 T = TypeVar("T")
